-- BEGIN COPYRIGHT
-- *************************************************************************
-- Licensed Materials - Property of IBM 
-- 5724-L01, 5655-N53, 5724-I82, 5655-R15
-- (C) Copyright IBM Corporation 2006, 2009. All rights reserved. 
-- US Government Users Restricted Rights - Use, duplication, or disclosure
-- restricted by GSA ADP Schedule Contract with IBM Corp.
-- *************************************************************************
-- END COPYRIGHT
--

----------------------------------------------------------------------
-- Following variables needs to be changed for customization and 
-- before running this script.
-- @STOGRP@ = DB2 Storage group name
-- @DB_NAME@         = DB2 Database name
-- @SCHEMA@          = DB2 SQLID (Schema Qualifier)
----------------------------------------------------------------------

-- *******************************************
-- Create dynamic artifact repository tables
-- for DB2 on zOS
-- *******************************************

--
-- *******************************************
-- Drop existing tablespaces
-- *******************************************

-- *******************************************
-- Create tablespaces
-- *******************************************
CREATE TABLESPACE D2D
IN @DB_NAME@ USING STOGROUP @STOGRP@
 CCSID UNICODE
 DEFINE NO
 SEGSIZE 32
 LOCKSIZE ANY
 BUFFERPOOL @BPTABLE4K@;
 
 CREATE LOB TABLESPACE D2DLOG
 IN @DB_NAME@
 USING STOGROUP @STOGRP@
 LOCKSIZE LOB
 BUFFERPOOL @BPLOB4K@;

create table @SCHEMA@.d2d_item (
	id varchar(255) not null, 
	state varchar(20) not null,
	item_level int not null,
	module_name varchar(255),
	application_name varchar(255),
	version varchar(30), 
	wps_version varchar(10),
	new_app_name varchar(255),
	primary key (id))
	IN @DB_NAME@.D2D;

CREATE UNIQUE INDEX @SCHEMA@.D2DITEM_INDEX_PK 
ON @SCHEMA@.d2d_item (id)
 USING STOGROUP @STOGRP@
 DEFINE NO
 NOT PADDED
 BUFFERPOOL @BPINDEX@;

create table @SCHEMA@.d2d_progress (
	item_id varchar(255) not null,
	progress INTEGER,
	progress_msg varchar(255),
	progress_error_code  varchar(255),
	primary key (item_id))
	IN @DB_NAME@.D2D;

CREATE UNIQUE INDEX @SCHEMA@.D2DPROGRESS_INDEX_PK 
ON @SCHEMA@.d2d_progress (item_id)
 USING STOGROUP @STOGRP@
 DEFINE NO
 NOT PADDED
 BUFFERPOOL @BPINDEX@;
 
create table @SCHEMA@.d2d_content (
	item_id varchar(255) not null,
	pi	blob,
	primary key (item_id))
	IN @DB_NAME@.D2D;
	
CREATE UNIQUE INDEX @SCHEMA@.D2DCONTENT_INDEX_PK 
ON @SCHEMA@.d2d_content (item_id)
 USING STOGROUP @STOGRP@
 DEFINE NO
 NOT PADDED
 BUFFERPOOL @BPINDEX@;
 
 CREATE AUX TABLE @SCHEMA@.D2DCONTENTBLOB
 IN @DB_NAME@.D2DLOG STORES @SCHEMA@.d2d_content
 COLUMN pi;

CREATE INDEX @SCHEMA@.D2DCONTENTBLOB_IDX
 ON @SCHEMA@.D2DCONTENTBLOB
 USING STOGROUP @STOGRP@
 BUFFERPOOL @BPINDEX@;
 
create table @SCHEMA@.d2d_metadata(
	item_id varchar(255) not null,
	name varchar(255) not null,
	value varchar(255) )
	IN @DB_NAME@.D2D;

CREATE UNIQUE INDEX @SCHEMA@.D2DMETADATA_INDEX_PK 
ON @SCHEMA@.d2d_metadata (item_id, name, value)
 USING STOGROUP @STOGRP@
 DEFINE NO
 NOT PADDED
 BUFFERPOOL @BPINDEX@;
 
create table @SCHEMA@.d2d_lock(
	item_lock character(1),
	lastUpdate timestamp)
	IN @DB_NAME@.D2D;

insert into @SCHEMA@.d2d_lock values ('x', null);

create table @SCHEMA@.d2d_message(
	item_id varchar(255) not null,
	seq int not null,
	id	varchar(63),
	description varchar(1024),
	affected_resource varchar(255),
	location varchar(63),
	folder varchar(255),
	chr varchar(63),
	tag	varchar(63),
	severity varchar(15),
	time timestamp,
	primary key (item_id, seq))
	IN @DB_NAME@.D2D;
	
CREATE UNIQUE INDEX @SCHEMA@.D2DMESSAGE_INDEX_PK 
ON @SCHEMA@.d2d_message (item_id, seq)
 USING STOGROUP @STOGRP@
 DEFINE NO
 NOT PADDED
 BUFFERPOOL @BPINDEX@;

insert into @SCHEMA@.schema_status values ('DirectDeploy', '7.5.0', '7.5.0');