# BEGIN COPYRIGHT
# *************************************************************************
# Licensed Materials - Property of IBM 
# 5724-L01, 5655-N53, 5724-I82, 5655-R15
# Copyright IBM Corporation 2008. All rights reserved. 
# US Government Users Restricted Rights - Use, duplication, or disclosure
# restricted by GSA ADP Schedule Contract with IBM Corp.
# *************************************************************************
# END COPYRIGHT

import os
import re

def getPropertyValue( propertyName ):
	framework.log('info', 'SCAWLEUpgrade', 'getPropertyValue', 'entering propertyName = ' + propertyName)
	propertyValue = java.lang.System.getProperty( propertyName )
	if( propertyValue == None ):
		framework.log('info', 'SCAWLEUpgrade', 'getPropertyValue','exiting')
		return ''
	framework.log('info', 'SCAWLEUpgrade', 'getPropertyValue','exiting ' + propertyValue)
	return propertyValue
	
#------------------------------------------------------------------------------
def createSIBDestination( busName, nodeName, serverName, clusterName, destName ):
	framework.log('info', 'SCAWLEUpgrade', 'createSIBDestination', 'Entering')
	parameters = busName + ' ' + nodeName + ' ' + serverName + ' ' + clusterName + ' ' + destName
	framework.log('info', 'SCAWLEUpgrade', 'createSIBDestination', parameters)
	
	if( clusterName == '' ):
		try:
			AdminTask.createSIBDestination( '[-bus ' + busName + ' -name ' + destName + ' -type QUEUE ' + '-node ' + nodeName + ' -server ' + serverName + ']' )
		except:
			framework.log('error', 'SCAWLEUpgrade', 'createSIBDestination', 'Exiting 0, error cocurred when trying to createSIBDestination')
			return 0
		else:
			framework.log('info', 'SCAWLEUpgrade', 'createSIBDestination', 'Exiting 1, create SIBDestination successful')
			return 1
	else:
		try:
			AdminTask.createSIBDestination( '[-bus ' + busName + ' -name ' + destName + ' -type QUEUE' + ' -cluster ' + clusterName + ']' )
		except:
			framework.log('error', 'SCAWLEUpgrade', 'createSIBDestination', 'Exiting 0, error cocurred when trying to createSIBDestination')
#			return 0
# TODO for some reason, the AppCluster queues seem to already exist, and it's causing failures here
			framework.log('error', 'SCAWLEUpgrade', 'createSIBDestination', 'Exiting 1, attempting to continue')
			return 1
		else:
			framework.log('info', 'SCAWLEUpgrade', 'createSIBDestination', 'Exiting 1, create SIBDestination successful')
			return 1
	framework.log('info', 'SCAWLEUpgrade', 'createSIBDestination', 'Exiting')

def lookupMELocation( nodeName, serverName, clusterName ):
	framework.log('info', 'SCAWLEUpgrade', 'lookupMELocation', 'Entering')
	parameters = nodeName + ' ' + serverName + ' ' + clusterName
	framework.log('info', 'SCAWLEUpgrade', 'lookupMELocation', parameters)

	if( clusterName == '' ):
		cell = AdminConfig.list( 'Cell' )
		nodeList = AdminConfig.list( 'Node', cell )
		nodeList = wsadminToList( nodeList )
		for node in nodeList:
			currentNodeName = AdminConfig.showAttribute( node, 'name' )
			if( currentNodeName == nodeName ):
				serverList = AdminConfig.list( 'Server', node )
				serverList = wsadminToList( serverList )
				for server in serverList:
					currentServerName = AdminConfig.showAttribute( server, 'name' )
					framework.log('info', 'SCAWLEUpgrade', 'lookupMELocation','currentServerName is: ' + currentServerName)
					if( currentServerName == serverName ):
						scopelist = [server, node, cell]
						for scope in scopelist:
							variableMAPList = AdminConfig.list( 'VariableMap', scope )
							variableMAPList = wsadminToList( variableMAPList )
							variableMap = variableMAPList[0]
							entryList = AdminConfig.showAttribute( variableMap, 'entries' )
							entryList = wsadminToList( entryList )
							for entry in entryList:
								symbolicName = AdminConfig.showAttribute( entry, 'symbolicName' )
								if( symbolicName == 'SCA_REMOTE_DESTINATION_LOCATION' ):
									value = AdminConfig.showAttribute( entry, 'value' )
									framework.log('info', 'SCAWLEUpgrade', 'lookupMELocation', 'Exiting lookupMELocation ' + str(value))
									return value
		framework.log('info', 'SCAWLEUpgrade', 'lookupMELocation', 'Exiting')
		return ''
	else:
		cell = AdminConfig.list( 'Cell' )
		clusters = AdminConfig.list( 'ServerCluster' )
		clusters = wsadminToList( clusters )
		framework.log('info', 'SCAWLEUpgrade', 'lookupMELocation', 'Clusters is ' + ''.join(clusters))
		for cluster in clusters:
			currentClusterName = AdminConfig.showAttribute( cluster, 'name' )
			framework.log('info', 'SCAWLEUpgrade', 'lookupMELocation', 'CurrentClusterName is ' + currentClusterName)
			if( currentClusterName == clusterName ):
				variableMAPList = AdminConfig.list( 'VariableMap', cluster )
				variableMAPList = wsadminToList( variableMAPList )
				variableMap = variableMAPList[0]
				entryList = AdminConfig.showAttribute( variableMap, 'entries' )
				entryList = wsadminToList( entryList )
				for entry in entryList:
					symbolicName = AdminConfig.showAttribute( entry, 'symbolicName' )
					if( symbolicName == 'SCA_REMOTE_DESTINATION_LOCATION' ):
						value = AdminConfig.showAttribute( entry, 'value' )
						framework.log('info', 'SCAWLEUpgrade', 'lookupMELocation', 'Exiting ' + str(value))
						return value
		framework.log('info', 'SCAWLEUpgrade', 'lookupMELocation', 'Exiting')
		return ''
	framework.log('info', 'SCAWLEUpgrade', 'lookupMELocation', 'Exiting')

def createASForStandAlone( activationSpecName, activationSpecJndiName, messageListenerType, busName, destinationName, messageSelector ):
	framework.log('info', 'SCAWLEUpgrade', 'createASForStandAlone', 'Entering')
	parameters = messageListenerType + ' ' + busName + ' ' + destinationName + ' ' + messageSelector
	framework.log('info', 'SCAWLEUpgrade', 'createASForStandAlone', parameters)

	if( findASForStandAlone(activationSpecName) != 1 ):
		aliasName = 'SCA_Auth_Alias';
		resourceAdapterList = AdminConfig.list( 'J2CResourceAdapter' )
		resourceAdapterList = wsadminToList( resourceAdapterList )
		resourceAdapter = resourceAdapterList[0]
		framework.log('info', 'SCAWLEUpgrade', 'createASForStandAlone', 'Resource adapter is ' + resourceAdapter)
		params = '[-name ' + activationSpecName + ' -jndiName ' + activationSpecJndiName + ' -messageListenerType ' + messageListenerType + ' -authenticationAlias ' + aliasName + ']'
		framework.log('info', 'SCAWLEUpgrade', 'createASForStandAlone', 'parameters ' + params)
		try:
			AdminTask.createJ2CActivationSpec( resourceAdapter, params )
			framework.log('info', 'SCAWLEUpgrade', 'createASForStandAlone', 'J2CAS created successfully')
			destinationTypeId = AdminConfig.getid ( '/J2CActivationSpec:' + activationSpecName + '/J2EEResourceProperty:destinationType/' )
			framework.log('info', 'SCAWLEUpgrade', 'createASForStandAlone', 'destinationTypeId ' + destinationTypeId)
			AdminConfig.modify( destinationTypeId, [['value', 'Queue']] )  #need to check
			destinationNameId = AdminConfig.getid( '/J2CActivationSpec:' + activationSpecName + '/J2EEResourceProperty:destinationName/' )
			AdminConfig.modify( destinationNameId, [['value', destinationName]] )
			framework.log('info', 'SCAWLEUpgrade', 'createASForStandAlone', 'destinationNameId ' + destinationNameId)
			busNameId = AdminConfig.getid( '/J2CActivationSpec:' + activationSpecName + '/J2EEResourceProperty:busName/' )

			framework.log('info', 'SCAWLEUpgrade', 'createASForStandAlone', 'busNameId ' + busNameId)
			AdminConfig.modify( busNameId, [['value', busName]] )

			messageSelectorID = AdminConfig.getid( '/J2CActivationSpec:' + activationSpecName + '/J2EEResourceProperty:messageSelector/' )
			AdminConfig.modify( messageSelectorID, [['value', messageSelector]] )
		except:
			framework.log('error', 'SCAWLEUpgrade', 'createASForStandAlone', 'Exiting 0, create AS for standalone failed')
			return 0
		else:
			#AdminConfig.save()
			framework.log('info', 'SCAWLEUpgrade', 'createASForStandAlone', 'Exiting 1, created AS for StandAlone')
			return 1
	framework.log('info', 'SCAWLEUpgrade', 'createASForStandAlone', 'Exiting 1')
	return 1

def createASForNDServer( activationSpecName, activationSpecJndiName, messageListenerType, busName, destinationName, messageSelector, nodeName, serverName, clusterName ):
	framework.log('info', 'SCAWLEUpgrade', 'createASForNDServer', 'Entering')
	framework.log('info', 'SCAWLEUpgrade', 'createASForNDServer', nodeName + ' ' + serverName + ' ' + clusterName)

	aliasName = 'SCA_Auth_Alias';
	if( clusterName == '' ):
		cell = AdminConfig.list( 'Cell' )
		nodeList = AdminConfig.list( 'Node', cell )
		nodeList = wsadminToList( nodeList )
		for node in nodeList:
			currentNodeName = AdminConfig.showAttribute( node, 'name' )
			if( currentNodeName == nodeName ):
				serverList = AdminConfig.list( 'Server', node )
				serverList = wsadminToList( serverList )
				for server in serverList:
					currentServerName = AdminConfig.showAttribute( server, 'name' )
					framework.log('info', 'SCAWLEUpgrade', 'createASForNDServer', 'Current server ' + currentServerName)
					if( currentServerName == serverName ):
						if( findAS( server, '', activationSpecName ) != 1 ):
							framework.log('info', 'SCAWLEUpgrade', 'createASForNDServer', 'The AS does not exist')
							ralist = AdminConfig.list( 'J2CResourceAdapter', server )
							ralist = wsadminToList( ralist )
							framework.log('info', 'SCAWLEUpgrade', 'createASForNDServer', 'Resource adapters on server' + ''.join(ralist))
							if( ralist == [] ):
								framework.log('info', 'SCAWLEUpgrade', 'createASForNDServer', 'Exiting 0, J2CResourceAdapter does not exist on the server')								
								return 0
							else:
								for J2Cra in ralist:
									resourceAdapterList = AdminConfig.list( 'ResourceAdapter', J2Cra )
									resourceAdapterList = wsadminToList( resourceAdapterList )
									framework.log('info', 'SCAWLEUpgrade', 'createASForNDServer', 'Resource adapters ' + ''.join(resourceAdapterList))
									for ra in resourceAdapterList:
										raclass = AdminConfig.showAttribute( ra, 'resourceAdapterClass' )
										if( raclass == 'com.ibm.ws.sib.ra.inbound.impl.SibRaResourceAdapterImpl' ):
											params = '[-name ' + activationSpecName + ' -jndiName ' + activationSpecJndiName + ' -messageListenerType ' + messageListenerType + ' -authenticationAlias ' + aliasName + ']'
											framework.log('info', 'SCAWLEUpgrade', 'createASForNDServer', 'Params is ' + params)
											try:
												J2CASpecID = AdminTask.createJ2CActivationSpec( J2Cra, params )
												framework.log('info', 'SCAWLEUpgrade', 'createASForNDServer', 'AS created successfully')
												update( J2CASpecID, busName, destinationName, messageSelector )
												framework.log('info', 'SCAWLEUpgrade', 'createASForNDServer', 'Updated AS attribute successfully')
											except:
												framework.log('error', 'SCAWLEUpgrade', 'createASForNDServer', 'Exiting 0, create AS or update AS attribute failed')												
												return 0
											else:
												framework.log('info', 'SCAWLEUpgrade', 'createASForNDServer', 'Exiting 1')
												return 1
						else:
							framework.log('info', 'SCAWLEUpgrade', 'createASForNDServer', 'Exiting 1, the AS exists on the server')							
							return 1
		framework.log('info', 'SCAWLEUpgrade', 'createASForNDServer', 'Exiting 0')
		return 0
	else:
		clusterID = AdminConfig.getid( '/ServerCluster:' + clusterName )
		framework.log('info', 'SCAWLEUpgrade', 'createASForNDServer', 'clusterId is ' + clusterID)
		if( findAS( '', clusterID, activationSpecName ) != 1 ):
			ralist = AdminConfig.list( 'J2CResourceAdapter', clusterID )
			ralist = wsadminToList( ralist )
			if( ralist == [] ):
				framework.log('info', 'SCAWLEUpgrade', 'createASForNDServer', 'Exiting the J2CResourceAdapter does not existed on the cluster')				
				return
			else:
				for J2Cra in ralist:
					resourceAdapterList = AdminConfig.list( 'ResourceAdapter', J2Cra )
					resourceAdapterList = wsadminToList( resourceAdapterList )
					framework.log('info', 'SCAWLEUpgrade', 'createASForNDServer', ''.join(resourceAdapterList))
					for ra in resourceAdapterList:
						raclass = AdminConfig.showAttribute( ra, 'resourceAdapterClass' )
						if( raclass == 'com.ibm.ws.sib.ra.inbound.impl.SibRaResourceAdapterImpl' ):
							params = '[-name ' + activationSpecName + ' -jndiName ' + activationSpecJndiName + ' -messageListenerType ' + messageListenerType + ' -authenticationAlias ' + aliasName + ']'
							try:
								J2CASpecID = AdminTask.createJ2CActivationSpec( J2Cra, params )
								framework.log('info', 'SCAWLEUpgrade', 'createASForNDServer', 'Created AS successfully')
								update( J2CASpecID, busName, destinationName, messageSelector )
								framework.log('info', 'SCAWLEUpgrade', 'createASForNDServer','Updated AS successfully')
							except:
								framework.log('error', 'SCAWLEUpgrade', 'createASForNDServer',' Exiting 0, create AS or update AS attribute failed')
								return 0
							else:
								framework.log('info', 'SCAWLEUpgrade', 'createASForNDServer', 'Exiting 1')
								return 1
		else:
			framework.log('info', 'SCAWLEUpgrade', 'createASForNDServer', 'Exiting 1, the AS exists on the target cluster')
			return 1
		framework.log('info', 'SCAWLEUpgrade', 'createASForNDServer', 'Exiting 0')
		return 0
	framework.log('info', 'SCAWLEUpgrade', 'createASForNDServer', 'Exiting')

def update( J2CASpecID, busName, destinationName, messageSelector ):
	framework.log('info', 'SCAWLEUpgrade', 'update', 'Entering')
	parameters = J2CASpecID + ' ' + busName + ' ' + destinationName + ' ' + messageSelector
	framework.log('info', 'SCAWLEUpgrade', 'update', parameters)

	j2eeResouceProperties = AdminConfig.showAttribute( J2CASpecID, 'resourceProperties' )
	j2eeResouceProperties = wsadminToList( j2eeResouceProperties )
	for j2eeResouceProperty in j2eeResouceProperties:
		name = AdminConfig.showAttribute( j2eeResouceProperty, 'name' )
		if( name == 'destinationType' ):
			AdminConfig.modify( j2eeResouceProperty, [['value', 'Queue' ]] )
		elif( name == 'destinationName' ):
			AdminConfig.modify( j2eeResouceProperty, [['value', destinationName ]] )
		elif( name == 'busName' ):
			AdminConfig.modify( j2eeResouceProperty, [['value', busName ]] )
		elif( name == 'messageSelector' ):
			AdminConfig.modify( j2eeResouceProperty, [['value', messageSelector ]] )
	framework.log('info', 'SCAWLEUpgrade', 'update', 'Exiting')

def findAS( serverID, clusterID,  activationSpecName):
	framework.log('info', 'SCAWLEUpgrade', 'findAS', 'Entering ' + serverID + ' ' + clusterID + ' ' + activationSpecName)

	if( clusterID == '' ):
		asList = AdminConfig.list( 'J2CActivationSpec', serverID )
		asList = wsadminToList( asList )
		for a in asList:
			name = AdminConfig.showAttribute( a, 'name' )
			if( name == activationSpecName ):
				framework.log('info', 'SCAWLEUpgrade', 'findAS', 'Exiting 1, found AS')
				return 1
		framework.log('info', 'SCAWLEUpgrade', 'findAS', 'Exiting 0')
		return 0
	else:
		asList = AdminConfig.list( 'J2CActivationSpec', clusterID )
		asList = wsadminToList( asList )
		for a in asList:
			name = AdminConfig.showAttribute( a, 'name' )
			if( name == activationSpecName ):
				framework.log('info', 'SCAWLEUpgrade', 'findAS', 'Exiting 1, found AS')
				return 1
		framework.log('info', 'SCAWLEUpgrade', 'findAS', 'Exiting 0')
		return 0
	framework.log('info', 'SCAWLEUpgrade', 'findAS', 'Exiting')

def findASForStandAlone(activationSpecName):
	framework.log('info', 'SCAWLEUpgrade', 'findASForStandAlone', 'Entering')
	asID = AdminConfig.list( 'J2CActivationSpec' )
	asID = wsadminToList( asID )
	for a in asID:
		name = AdminConfig.showAttribute( a, 'name' )
		if( name == activationSpecName ):
			framework.log('info', 'SCAWLEUpgrade', 'findASForStandAlone', 'Exiting 1, found AS')
			return 1
	framework.log('info', 'SCAWLEUpgrade', 'findASForStandAlone', 'Exiting 0')
	return 0

def isSCAConfigured( nodeName, serverName, clusterName ):
	framework.log('info', 'SCAWLEUpgrade', 'isSCAConfigured', 'Entering ' + nodeName + ' ' +serverName + ' ' + clusterName)

	if( clusterName == '' ):
		try:
			configList = AdminTask.showSCAAsyncConfig( '[-nodeName ' + nodeName + ' -serverName ' + serverName + ']' )
		except:
			framework.log('info', 'SCAWLEUpgrade', 'isSCAConfigured', 'Exiting 0, SCA is not configured on server')
			return 0
		else:
			if( configList.find( '[configured true]' ) > 0 ):
				framework.log('info', 'SCAWLEUpgrade', 'isSCAConfigured', 'Exiting 1')
				return 1
			else:
				framework.log('info', 'SCAWLEUpgrade', 'isSCAConfigured', 'Exiting 0, SCA is not configured on server')
				return 0
	else:
		try:
			configList = AdminTask.showSCAAsyncConfig( '[-clusterName ' + clusterName + ']' )
		except:
			framework.log('error', 'SCAWLEUpgrade', 'isSCAConfigured', 'Exiting 0, SCA is not configured on server')
			return 0
		else:
			if( configList.find( '[configured true]' ) > 0 ):
				framework.log('info', 'SCAWLEUpgrade', 'isSCAConfigured', 'Exiting 1')
				return 1
			else:
				framework.log('info', 'SCAWLEUpgrade', 'isSCAConfigured', 'Exiting 0')
				return 0
	framework.log('info', 'SCAWLEUpgrade', 'isSCAConfigured', 'Exiting')

def parseNode( remoteME ):
	framework.log('info', 'SCAWLEUpgrade', 'parseNode', 'Entering ' + remoteME)
	
	if( remoteME.find( 'node=' ) != -1 ) and ( remoteME.find( 'server=' ) != -1 ):
		begin = remoteME.find( 'node=' )
		begin = begin + len( 'node=' )
		end = remoteME.find( ',server=' )
		node = remoteME[begin:end]
		framework.log('info', 'SCAWLEUpgrade', 'parseNode', 'Exiting ' + node)
		return node
	else:
		framework.log('info', 'SCAWLEUpgrade', 'parseNode', 'Exiting')
		return ''

def parseServer( remoteME ):
	framework.log('info', 'SCAWLEUpgrade', 'parseServer', 'Entering ' + remoteME)
	if( remoteME.find( 'node=' ) != -1 ) and ( remoteME.find( 'server=' ) != -1 ):
		begin = remoteME.find( 'server=' )
		begin = begin + len( 'server=' )
		server = remoteME[begin:len( remoteME )]
		framework.log('info', 'SCAWLEUpgrade', 'parseServer', 'Exiting ' + server)
		return server
	else:
		framework.log('info', 'SCAWLEUpgrade', 'parseServer', 'Exiting')
		return ''

def parseCluster( remoteME ):
	framework.log('info', 'SCAWLEUpgrade', 'parseCluster', 'Entering ' + remoteME)
	
	if( remoteME.find( 'cluster=' ) != -1 ):
		cluster = remoteME[18:len( remoteME )]
		framework.log('info', 'SCAWLEUpgrade', 'parseCluster', 'Exiting ' + cluster)
		return cluster
	else:
		framework.log('info', 'SCAWLEUpgrade', 'parseClsuter', 'Exiting')
		return ''

# returns "1" if and only if |cluster| is the messaging cluster for the deployment environment
def isMessagingCluster ( cluster ):
	framework.log('info', 'SCAWLEUpgrade', 'isMessagingCluster', 'Entering ' + cluster)
	
	# this returns the cell of the dmgr to which the script is connected
	cell = AdminConfig.list( 'Cell' )
	framework.log('info', 'SCAWLEUpgrade', 'isMessagingCluster', 'Cell is ' + cell)
	wsadminToList( cell )
	# cell-scope search
	variableMapList = AdminConfig.list( 'VariableMap', cell )
	variableMapList = wsadminToList( variableMapList )
	for variableMap in variableMapList:
		entryList = AdminConfig.showAttribute( variableMap, 'entries' )
		entryList = wsadminToList( entryList )
		for entry in entryList:
			symbolicName = AdminConfig.showAttribute( entry, 'symbolicName' )
			if ( symbolicName == "SCA_REMOTE_DESTINATION_LOCATION" ):
				messagingCluster = AdminConfig.showAttribute( entry, 'value' )
				framework.log('info', 'SCAWLEUpgrade', 'isMessagingCluster', 'Messaging cluster ' + messagingCluster)
				if ( messagingCluster == str( "WebSphere:cluster=" + cluster ) ):
					framework.log('info', 'SCAWLEUpgrade', 'isMessagingCluster', 'Exiting 1, this is messaging cluster')
					return 1
				else:
					# we know we're not the messaging cluster
					framework.log('info', 'SCAWLEUpgrade', 'isMessagingCluster', 'Exiting 0, this is not messaging cluster')
					return 0
	# assume the lack of an appropriate variable means there is no separate messaging cluster
	framework.log('info', 'SCAWLEUpgrade', 'isMessagingCluster', 'Exiting 0, this is not messaging cluster')
	return 0

#------------------------------------------------------------------------------
def upgradeBPMPSProfile():
	framework.log('info', 'SCAWLEUpgrade', 'upgradeBPMPSProfile', 'Entering')

	# 1. If update required, perform update
	#1.1 Initialization
	node = getUpdatedNode();
	framework.log('info', 'SCAWLEUpgrade', 'upgradeBPMPSProfile', 'Node is ' + node)
	cluster = getUpdatedCluster()
	framework.log('info', 'SCAWLEUpgrade', 'upgradeBPMPSProfile', 'Cluster is ' + cluster)
	#1.2 If node update is performed
	if( cluster == '' ):
		serverList = getServerList( cell, node )
		framework.log('info', 'SCAWLEUpgrade', 'upgradeBPMPSProfile', 'Server list is ' + ''.join(serverList))
		for server in serverList:
			framework.log('info', 'SCAWLEUpgrade', 'upgradeBPMPSProfile', 'Server is ' + server)
			nodeID = AdminConfig.getid( '/Cell:' + cell + '/Node:' + node )
			framework.log('info', 'SCAWLEUpgrade', 'upgradeBPMPSProfile', 'Node id is ' + nodeID)
			serverID = AdminConfig.getid( '/Cell:' + cell + '/Node:' + node + '/Server:' + server )
			framework.log('info', 'SCAWLEUpgrade', 'upgradeBPMPSProfile', 'Server id is ' + serverID)
			if( isSCAConfigured( node, server, '' ) ):
				framework.log('info', 'SCAWLEUpgrade', 'upgradeBPMPSProfile', 'SCA is configured on the server')
				queueID = 'sca/WLE_' + node + '_' + server
				framework.log('info', 'SCAWLEUpgrade', 'upgradeBPMPSProfile', 'Queue id is ' + queueID)
				destination = findDestination( scaBusName, queueID )
				if( destination == '' ):					
					framework.log('info', 'SCAWLEUpgrade', 'upgradeBPMPSProfile', 'The destination ' + queueID + ' do not exist')
					remoteME = lookupMELocation( node, server, '' )
					meNode = node
					meServer = server
					meCluster = cluster
					if( remoteME != '' ):
						meNode = parseNode( remoteME )
						meServer = parseServer( remoteME )
						meCluster = parseCluster( remoteME )
					else:
						meNode = node
						meServer = server
						meCluster = cluster
					if( meCluster == '' ):
						if( createSIBDestination( scaBusName, meNode, meServer, '', queueID ) == 0 ):
							continue
					else:
						if( createSIBDestination( scaBusName, '', '', meCluster, queueID ) == 0 ):
							continue	
				activationSpecName = 'SCA_WLE_' + node + '_' + server + '_AS'
				activationSpecJndiName = 'sca/WLE_' + node + '_' + server + '/ActivationSpec'			
				if( nodeType == 'unmanaged' ):					
					if( createASForStandAlone( activationSpecName, activationSpecJndiName, messageListenerType, scaBusName, queueID, messageSelector ) == 0 ):
						continue
				elif( nodeType == 'managed' ):
					if( createASForNDServer( activationSpecName, activationSpecJndiName, messageListenerType, scaBusName, queueID, messageSelector, node, server, '' ) == 0 ):
						continue

				AdminConfig.save()
				framework.log('info', 'SCAWLEUpgrade', 'upgradeBPMPSProfile', 'Upgrade of node and server successful')

	#1.3 If cluster update is performed
	else:
		if( isSCAConfigured( '', '', cluster ) ):
			if ( isMessagingCluster( cluster ) == 0 ):
				framework.log('info', 'SCAWLEUpgrade', 'upgradeBPMPSProfile', 'Cluster has SCA configured and is not a messaging cluster')
				queueID = 'sca/WLE_' + cluster
				destination = findDestination( scaBusName, queueID )
				if(destination == ''):
					clusterID = AdminConfig.getid( '/Cell:' + cell + '/ServerCluster:' + cluster )
					remoteME = lookupMELocation( '', '', cluster )
					meNode = ''
					meServer = ''
					meCluster = cluster
					framework.log('info', 'SCAWLEUpgrade', 'upgradeBPMPSProfile', 'Cluster is ' + cluster)
					if( remoteME != '' ):
						meNode = parseNode( remoteME )
						meServer = parseServer( remoteME )
						meCluster = parseCluster( remoteME )
					if( meCluster == '' ):
						if( createSIBDestination( scaBusName, meNode, meServer, '', queueID ) == 0 ):
							framework.log('info', 'SCAWLEUpgrade', 'upgradeBPMPSProfile', 'Exiting 0')
							return 0
					else:
						if( createSIBDestination( scaBusName, '', '', meCluster, queueID ) == 0 ):
							framework.log('info', 'SCAWLEUpgrade', 'upgradeBPMPSProfile', 'Exiting 0')
							return 0
				activationSpecName = 'SCA_WLE_' + cluster + '_AS'
				activationSpecJndiName = 'sca/WLE_' + cluster + '/ActivationSpec'
				if( createASForNDServer( activationSpecName, activationSpecJndiName, messageListenerType, scaBusName, queueID, messageSelector, '', '', cluster ) == 0 ):
					framework.log('info', 'SCAWLEUpgrade', 'upgradeBPMPSProfile', 'Exiting 0')
					return 0
				AdminConfig.save()
				framework.log('info', 'SCAWLEUpgrade', 'upgradeBPMPSProfile', 'Cluster upgraded successfully')
			else:
				framework.log('info', 'SCAWLEUpgrade', 'upgradeBPMPSProfile', 'Cluster is a messaging cluster, SCA-WLE will not be configured')
		else:
			framework.log('info', 'SCAWLEUpgrade', 'upgradeBPMPSProfile', 'Cluster is not SCA-enabled, SCA-WLE will not be configured')
	framework.log('info', 'SCAWLEUpgrade', 'upgradeBPMPSProfile', 'Exiting')

def wsadminToList( inStr ):
	framework.log('info', 'SCAWLEUpgrade', 'wsadminToList','entering ' + inStr)	
	outList = []
	if ( len( inStr ) > 0 and inStr[0] == '[' and inStr[-1] == ']' ):
		inStr = inStr[1:-1]
		tmpList = inStr.split( " " )
	else:
		tmpList = inStr.split( "\n" )	#splits for Windows or Linux
	for item in tmpList:
		item = item.rstrip();				#removes any Windows "\r"
		if ( len( item ) > 0 ):
			outList.append( item )
	framework.log('info', 'SCAWLEUpgrade', 'wsadminToList','exiting ' + ''.join(outList))	
	return outList
	
def getCell():
	framework.log('info', 'SCAWLEUpgrade', 'getCell','entering')
	cellName = '';
	cellList = wsadminToList( AdminConfig.list( 'Cell' ) )
	for cell in cellList:
		cellName = AdminConfig.showAttribute( cell, 'name' )
	framework.log('info', 'SCAWLEUpgrade', 'getCell', 'exiting ' + cellName)
	return cellName

#------------------------------------------------------------------------------
# get the current node name which is migrated
#------------------------------------------------------------------------------
def getUpdatedNode():
	framework.log('info', 'SCAWLEUpgrade', 'getUpdatedNode','entering')
	ret = getPropertyValue( 'local.node' )
	framework.log('info', 'SCAWLEUpgrade', 'getUpdatedNode','exiting ' + ret)
	return ret

def getUpdatedCluster():
	framework.log('info', 'SCAWLEUpgrade', 'getUpdatedCluster','entering')
	ret = getPropertyValue( 'wbi.upgrade.cluster.name' )
	framework.log('info', 'SCAWLEUpgrade', 'getUpdatedCluster','exiting ' + ret)
	return ret

def getServerList( cell, node ):
	framework.log('info', 'SCAWLEUpgrade', 'getServerList','entering cell ' + cell + ' node ' + node)
	serverNameList = []
	nodeId = AdminConfig.getid( '/Cell:' + cell + '/Node:' + node )
	if( nodeId == '' ):
		framework.log('info', 'SCAWLEUpgrade', 'getServerList','exiting ' + ''.join(serverNameList))
		return serverNameList
	serverList = wsadminToList( AdminConfig.list( 'Server', nodeId ) )
	for server in serverList:
		serverName = AdminConfig.showAttribute( server, 'name' )
		clusterName = AdminConfig.showAttribute( server, 'clusterName' )
		if( clusterName == None ):
			if( serverName != 'nodeagent' ):
				serverNameList.append( serverName )
	framework.log('info', 'SCAWLEUpgrade', 'getServerList','exiting ' + ''.join(serverNameList))
	return serverNameList
	
def findDestination( scabusName, queueId ):
	framework.log('info', 'SCAWLEUpgrade', 'findDestination', 'Entering ' + scabusName + ' ' + queueId)

	busList = AdminConfig.list( 'SIBus' )
	busList = wsadminToList( busList )
	for bus in busList:
		busName = AdminConfig.showAttribute( bus, 'name' )
		if( busName == scabusName ):
			destinationList = AdminConfig.list( 'SIBQueue', bus )
			destinationList = wsadminToList( destinationList )
			for destination in destinationList:
				destinationID = AdminConfig.showAttribute( destination, 'identifier' )
				if( destinationID == queueId ):
					framework.log('info', 'SCAWLEUpgrade', 'findDestination',	'The destination ' + destinationID + ' already exists')
					return destination
	return ''
	framework.log('info', 'SCAWLEUpgrade', 'findDestination',	'exiting ')
		
#main		
framework = BPMMigrationFramework()
profileType = getPropertyValue( 'wbi.upgrade.profile.type' )
cell = getCell()
scaBusName = 'SCA.SYSTEM.' + cell + '.Bus'
messageListenerType = 'com.ibm.wsspi.sib.ra.SibRaMessageListener'
messageSelector = '(SI_CorrelationID IS NULL) AND (user.scaStoredMsgId IS NULL)'
nodeType = getPropertyValue( 'wbi.upgrade.node.type' )
if( profileType == 'WPS'):	
	if( nodeType == 'dmgr' or nodeType == 'unmanaged'):
		upgradeBPMPSProfile()
else:
	framework.log('info', 'SCAWLEUpgrade', 'SCAWLEUpgrade', 'This is not a BPM profile hence no WLE-SCA migration needed')

